import logging
import os
import subprocess
import sys
import webbrowser
from pathlib import Path

ROOT_DIR = Path(__file__).parent.absolute()
_log = logging.getLogger(__name__)

if sys.platform.startswith("win32"):
    pass
elif sys.platform.startswith("linux"):
    OS_OPEN = "xdg-open"
# Linux-specific code here...
elif sys.platform.startswith("darwin"):
    OS_OPEN = "open"
else:
    OS_OPEN = None


def open_it(uri: str):
    if OS_OPEN is None:
        _log.error(f"Unknown OS architecture: {sys.platform}")
        return

    _log.debug(f"{uri=}")
    p = Path.home()  # default setting

    if uri.startswith("shell::"):
        cmd = uri.removeprefix("shell::")
        run_it(cmd)
        return
    if uri.startswith("http"):
        _log.debug(f"Opening HTTP Link")
        # p = uri
        webbrowser.open(uri, new=2)
        return
    elif uri[0] in "/,.,~,$":
        if uri.startswith("/"):
            _log.debug(f"Absolute path.")
            p = Path(uri)
        elif uri.startswith("~"):
            _log.debug(f"Path with prefix tilde.")
            p = Path(uri).expanduser().absolute()
        elif uri.startswith("$"):
            _log.debug(f"Path with environment prefix.")
            p = Path(uri)
            env_path = os.getenv(p.parts[0].strip("$"), None)
            if env_path is None:
                _log.warning(f"{p.parts[0]} not set in environment. Cannot proceed.")
                return
            p = Path(env_path) / Path(*p.parts[1:])
        elif uri.startswith("."):
            _log.debug(f"Relative path: {uri}, working dir: {os.getcwd()}")
            p = Path(uri).absolute()

        if not p.exists():
            _log.warning(f"{p} does not exists.")
            return
    else:
        _log.warning(f"Unknown protocol: {uri=}")
        return

    _log.info(f"Opening: {p}")
    subprocess.run([OS_OPEN, p])


def run_it(cmd: str):
    """
    better shell handling:
    s = shlex(cmd, posix=True, punctuation_chars=True)
    """
    _log.info(f"{cmd=}")
    subprocess.run(cmd, shell=True)


if __name__ == "__main__":
    # cmd = "vim +/'## SqlAlchemy' ../test/tests_data/sample_docu.md"
    # cmd = "vim +/'## SqlAlchemy' /Users/Q187392/dev/py/twbm/test/tests_data/sample_docu.md"
    # cmd = "vim +/'## SqlAlchemy' $HOME/dev/py/twbm/test/tests_data/sample_docu.md"
    cmd = "vim +/'## SqlAlchemy' ~/dev/py/twbm/test/tests_data/sample_docu.md"
    run_it(cmd)



import sqlite3
import aiosql

queries = aiosql.from_path("../sql/queries.sql", "sqlite3")

if __name__ == "__main__":
    # db_url = '../test/tests_data/test.db'
    db_url = "./bm2.db"
    conn = sqlite3.connect(db_url)
    print(queries)
    # r = queries.get_all(conn)
    # print(r)
    r = queries.create_db(conn)
    print(r)



"""Default environment
Edit service environment to override
"""
################################################################################
# Base Environment
################################################################################
from pathlib import Path

from pydantic import BaseSettings

ROOT_DIR = Path(__file__).parent.absolute()


class Environment(BaseSettings):
    log_level: str = "INFO"
    twbm_db_url: str = "sqlite:///db/bm.db"

    @property
    def dbfile(self):
        return f"{self.twbm_db_url.split('sqlite:///')[-1]}"


config = Environment()
_ = None



import pytest
from typer.testing import CliRunner

from twbm import app

runner = CliRunner()


class TestSearch:
    def test_search_d(self, dal):
        result = runner.invoke(app, ["search", "-v"], input="d 1 2\n")
        print(result.stdout)
        assert result.exit_code == 0
        assert "Index 4 deleted" in result.stdout
        assert "Index 3 deleted" in result.stdout

    def test_search_p(self, dal):
        result = runner.invoke(app, ["search", "-v"], input="p 1 2\n")
        print(result.stdout)
        assert result.exit_code == 0
        assert "xxxxx" in result.stdout

    def test_edit_all(self, mocker, dal):
        mocked = mocker.patch("twbm.twb.BukuDb.edit_update_rec")
        result = runner.invoke(app, ["search", "-v"], input="e\n")
        print(result.stdout)

        # than all table entries should be edited
        assert len(mocked.call_args_list) == 10

    def test_search_non_interactive(self, dal):
        tosearch = '"http://11111/11111"'
        result = runner.invoke(app, ["search", "-v", "--np", tosearch])
        print(result.stdout)
        assert result.exit_code == 0
        assert "11111" in result.stdout
        assert "5" in result.stdout  # print for pipeline

    def test_search_tags_exact(self, dal):
        result = runner.invoke(app, ["search", "-v", "--np", "-e", "aaa,bbb"])
        print(result.stdout)
        assert result.exit_code == 0
        assert "Found: 2" in result.stdout

    def test_search_tags_exact_none(self, dal):
        result = runner.invoke(app, ["search", "-v", "--np", "-e", "aaa"])
        print(result.stdout)
        assert result.exit_code == 0
        assert "Found: 0" in result.stdout

    def test_search_tags_exact_invalid(self, dal):
        result = runner.invoke(app, ["search", "-v", "--np", "-e", "aaa,", "bbb"])
        print(result.stdout)


@pytest.mark.skip("Interactive Tests.")
class TestHandleUri:
    def test_search_browser_should_open(self, dal):
        result = runner.invoke(app, ["search", "-v"], input="1 2\n")
        print(result.stdout)
        assert result.exit_code == 0

    def test_search_pptx_should_open(self, dal):
        result = runner.invoke(app, ["search", "-v", "pptx"], input="0\n")
        print(result.stdout)
        assert result.exit_code == 0

    @pytest.mark.skip("not working without terminal")
    def test_search_shell_exec(self, dal):
        result = runner.invoke(app, ["search", "-v", "shell open vim"], input="0\n")
        print(result.stdout)
        assert result.exit_code == 0


@pytest.mark.skip("Notnworking: not allowed operations: fileno()")
def test_upgrade(dal):
    result = runner.invoke(app, ["update", "-v", "-n", "aaa", "2,3"])
    print(result.stdout)
    assert result.exit_code == 0


def test_delete(dal):
    result = runner.invoke(app, ["delete", "-v", "6"])
    print(result.stdout)


class TestAddUrl:
    def test_add(self, dal):
        result = runner.invoke(
            app, ["add", "--title", "title1", "https://www.google.com"]
        )
        print(result.stdout)
        assert result.exit_code == 0

    def test_add_with_new_tags_yes(self, dal):
        result = runner.invoke(
            app,
            [
                "add",
                "--title",
                "title1",
                "https://www.google.com",
                "pa,pb",
                " pz",
                "PZ",
            ],
            input="y\n",
        )
        print(result.stdout)
        assert result.exit_code == 0

    def test_add_with_new_tags_no(self, dal):
        result = runner.invoke(
            app,
            [
                "add",
                "--title",
                "title1",
                "https://www.google.com",
                "pa,pb",
                " pz",
                "PZ",
            ],
            input="n\n",
        )
        print(result.stdout)
        assert "Create unknown_tags=['pa', 'pb', 'pz']" in result.stdout
        assert result.exit_code == 1


class TestTags:
    def test_tags(self, dal):
        result = runner.invoke(app, ["tags", "-v"])
        print(result.stdout)
        assert result.exit_code == 0

    def test_tags_related(self, dal):
        result = runner.invoke(app, ["tags", "-v", "xxx"])
        print(result.stdout)
        assert result.exit_code == 0
